#include <stdio.h>
#include <stdlib.h>
#include <dirent.h>
#include <sys/stat.h>
#include <pthread.h>
#include <signal.h>
#include "config.h"

int get_info(char *path)
{
    int end = 0;

    pthread_mutex_lock(&info.sig_parent); // all processes could block otherwise
    kill(parent,SIGHUP);            // signal parent so it knows we need a new dir
    sleep(1);                       // ensure parent gets the lock first
    pthread_mutex_lock(&info.mutex);

    // get the new working directory
    memset(path,0x0,2048);
    if(info.dir[0]) strcpy(path,info.dir);
    else end = 1;

    pthread_mutex_unlock(&info.mutex);
    pthread_mutex_unlock(&info.sig_parent);
    if(end) return 0;
   
    return 1;
}

void clone_main(void *arg)
{
 char path[MAXFILELEN]; // pthreads share all memory except stack
                        // during thread-change the cwd changes
 int count = 0;         // problems accur when we share these vars
 DIR *array[10];
 struct stat st;
 
 while(get_info(path) != 0)
 {
#ifdef DEBUG
    printf("[clone]: checking %s\n",path);
#endif
    count = 0;
    *((int *) arg) += 1;

    memset(&st,0x0,sizeof(struct stat));
    if(lstat(path,&st) == -1) 
    {
        printf("lstat failed on %s: %s\n",path,strerror(errno));
        continue;
    }
    
    if(checkaccess(&st) != -1)
    {
        array[count] = opendir(path);
        check_dir(array[count],&count,path,array);
    }
 }
}

void check_dir(DIR *root,int *count, char *path,DIR *array[10])
{
 struct dirent *next;
 struct stat st;
 char *strip;
 
 if(root == NULL) return;
 while((next = readdir(root)) != NULL)
 {
    /* This shouldn't be able to happen */
    if(strlen(path) + strlen(next->d_name) > MAXFILELEN)
    {
        printf("[clone]: path is bigger than %d\n",MAXFILELEN);
        continue;    
    }
 
    strcat(path,"/"); strcat(path,next->d_name);

#ifdef DEBUG
    printf("[clone]:  %s\n",path);
#endif

    memset(&st,0x0,sizeof(struct stat));
    if(lstat(path,&st) == -1) 
    {
        printf("lstat failed on %s: %s\n",path,strerror(errno));
        continue;
    }
           
    if((st.st_mode & S_IFMT) == S_IFDIR)
    {
        if(strcmp(next->d_name,".") && strcmp(next->d_name,".."))
	{
	    (*count)++;
	    if(*count > 10)
	    {
	      printf("Level deeper than 10, ignoring\n");
	      return;
	    }
            
            if(checkaccess(&st) != -1)
	    {
                array[*count] = opendir(path);
                check_dir(array[*count],count,path,array);
            }
            
	    (*count)--;
        }
    }else check_perms(&st,path);

    /* Cut off next->d_name */
    if(strip = (char *)strrchr(path,'/')) 
        *strip = 0x0;                   
 }
}


void check_perms(struct stat *sb,char *path)
{
  char perms[10], *p;
  int i, ok = 0;


  if(S_ISLNK(sb->st_mode)) return; /* don't parse links that causes all sorts of nasty shit */

  for(i = 0;i < 3;i++)
  {
        if((show[i] & READ) && (sb->st_mode & (S_IREAD >> i * 3))) 
               { ok = 1;break; }
                
        if((show[i] & WRITE) && (sb->st_mode & (S_IWRITE >> i * 3)))
               { ok = 1;break; }
                
        if((show[i] & EXEC) && (sb->st_mode & (S_IEXEC >> i * 3)))
               { ok = 1;break; }
  }

  if(show[3] && ((sb->st_mode & S_ISUID) || (sb->st_mode & S_ISGID)))
    ok = 1;

  if(ok)
  {
     p = perms + 1;
     strcpy(perms,"----------");

     for(i = 0;i < 3; i++)
     {
        if(sb->st_mode & (S_IREAD  >> i * 3)) *p = 'r';
        p++;
        if(sb->st_mode & (S_IWRITE >> i * 3)) *p = 'w';
        p++;
        if(sb->st_mode & (S_IEXEC  >> i * 3)) *p = 'x';
        p++;
     }
     if(sb->st_mode & S_ISUID) perms[3] = 's'; // suid
     if(sb->st_mode & S_ISGID) perms[6] = 's'; // sgid 
     if(sb->st_mode & S_ISVTX) perms[9] = 't'; // sticky bit

     /* Filetype */
     switch(sb->st_mode & S_IFMT)
     {
        case S_IFBLK: perms[0] = 'b';
                      break;
        case S_IFCHR: perms[0] = 'c';
                      break;
        case S_IFDIR: perms[0] = 'd';
                      break;
        case S_IFIFO: perms[0] = 'f';
                      break;
        default:      perms[0] = '-';
     }
     printf("%s %s\n",perms,path);
  }
}

int checkaccess(struct stat *sb)
{
    if(sb->st_uid == getuid() && (sb->st_mode & S_IXUSR|S_IRUSR))
        return 0; 
    else if(sb->st_gid == getgid() && (sb->st_mode & S_IXGRP|S_IRGRP))
        return 0; 
    else if(sb->st_mode & (S_IXOTH|S_IROTH))
        return 0;
    else return -1;
}

void perm2dig(char *perm,int kind)
{
    int tmp = 0;
    char *p;

    if(strlen(perm) > 3)
        fatal("permission '%s' is too big, please use something like 'rwx'\n",perm);

    p = perm;
    while(*p)
    {
        if(*p == 'r') tmp |= READ;
        if(*p == 'w') tmp |= WRITE;
        if(*p++ == 'x') tmp |= EXEC;
    }

    if(kind == 3) show[0] = tmp,show[1] = tmp,show[2] = tmp;
    else show[kind] = tmp;
}
