/* The file system spider by lamagra & qironlung*/
/* Includes */
#include <stdio.h>
#include <dirent.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <pthread.h>
#include "config.h"

/* Prototypes */
char *getdir();

/* Globals */
extern int errno;
DIR *root;
char *startdir = "/";   

/* Program functions */
void handler(int sig)
{
    char *tmp;
    pthread_mutex_lock(&info.mutex);
    
    tmp = getdir();
    if(tmp) strcpy(info.dir,tmp);
    else info.dir[0] = (char)NULL;
    
    pthread_mutex_unlock(&info.mutex);
}

void fatal(char *msg, ...)
{
   va_list ap;
   
   va_start(ap,msg);
   vfprintf(stderr,msg,ap);
   va_end(ap);
   exit(-1);
}

void usage(char *progname)
{
  printf("[parent]: Usage %s [options]\n",progname);
  printf("[parent] Options are:\n"
         "\t-d: the directory where the spider should start from\n"
         "\t-t: the number of threads to use (default = %d)\n"
         "\t-s: hide suid files\n"
         "\t-u: Change user permissions to shown like 'rwx' put - to show nothing\n"
         "\t-g: Change group permissions to show\n"
         "\t-o: Change other permissions to show\n"
         "\t-a: Change all permissions to show\n"
         "\t-h / -?: this thing\n"
        ,THREADDEFAULT);
  exit(-1);
}   

int main(int argc, char **argv)
{
   pthread_t id[THREADMAX];
   int i, counter[THREADMAX], thread = THREADDEFAULT;
   char ch;

   /* Set default permissions */
   show[0] = 0;
   show[1] = READ|WRITE;
   show[2] = READ|WRITE;
   show[3] = 1;

   while((ch = getopt(argc,argv,"h?t:sd:o:a:u:g:")) != EOF)
   {
        switch(ch)
        {
            case 't': thread = atoi(optarg);
                      // Sanity checks
                      if(thread < 1 || thread > THREADMAX)
                      {
                         printf("[parent]: Uhm dude, %d threads??\n",thread);
                         exit(-1);
                      }            
                      break;
                      
            case 'd': startdir = optarg;
                      break;

            case 's': show[3] = 0;
                      break;
                                            
            /* User perms */
            case 'u': perm2dig(optarg,0);
                      break;
                      
            /* Group perms */                      
            case 'g': perm2dig(optarg,1);
                      break;
                      
            /* Other perms */                      
            case 'o': perm2dig(optarg,2);
                      break;
                      
            /* All perms */          
            case 'a': perm2dig(optarg,3);
                      break;
                      
            case 'h':
            case '?': usage(argv[0]);
                      break;
                      
            default:  printf("Unknown options '%c', ignoring\n",ch);
        }
   }


   printf("[parent]: fs-spider by lamagra <access-granted@geocities.com>\n"
          "\tget new versions and other programs @ http://lamagra.seKure.de\n"
          "\tLamagra is a member of b0f/buffer0verfl0w security http:/www.b0f.com
          "\tstartdir = %s\n",startdir);
   printf("[parent]: Permissions %d / %d / %d\n",show[0],show[1],show[2]);

   signal(SIGHUP,handler);

   if(chdir(startdir) == -1)
	fatal("Couldn't chdir to %s: %s\n",startdir,strerror(errno));

   if((root = opendir(".")) == NULL) 
	fatal("Can't open %s: %s\n",startdir,strerror(errno));

   parent = getpid();

   for(i = 0; i < thread ;i++)
   {
     counter[i] = 0;
     pthread_create(&id[i],NULL,clone_main,&counter[i]);
#ifdef DEBUG
     printf("[parent]: started #%d\n",i);
#endif
   }

   for(i = 0;i < thread; i++)
   {
        pthread_join(id[i],NULL);
#ifdef DEBUG
        printf("[parent]: thread #%d finished\n",i);
#endif
        printf("[parent]: thread #%d got %d dirs\n",i,counter[i]);
   }
}

char *getdir()
{
   struct dirent *dir;
   struct stat st;
   static char tmp[1024];

   while(1)
   {
   	if((dir = readdir(root)) == NULL)
		return NULL;

        memset(tmp,0x0,1024);
        memset(&st,0x0,sizeof(struct stat));
   	strncpy(tmp,startdir,1024);
	if(startdir[strlen(startdir) - 1] != '/') strcat(tmp,"/");
   	strncat(tmp,dir->d_name,1024);

   	lstat(tmp,&st);
   	if(((st.st_mode & S_IFMT) == S_IFDIR) && strcmp(dir->d_name,".") 
            && strcmp(dir->d_name,"..")) 
                return tmp;
	else
            check_perms(&st,tmp);
   }
}

